////////////////////////////////////////////////////////////
// Experimental advanced shader class.
// This class is used to handle shaders used for 
// rendering various lit and textured 3D scene models.
//
// For this demo, this class is used to handle a simple
// single diffuse only texture shader that takes into
// account the direction and intensity of a single
// global directional light. 
//
// Version: 201212_01
////////////////////////////////////////////////////////////

#ifndef _ADVANCED_SHADER_H_
#define _ADVANCED_SHADER_H_

#include <d3d11.h>
#include <d3dx10math.h>
#include <d3dx11async.h>
#include <fstream>

using namespace std;

class AdvancedShader
{
public:
		AdvancedShader();
		~AdvancedShader();

		bool Initialize(ID3D11Device* device, HWND hwnd);
		bool Render(ID3D11DeviceContext* deviceContext, int indexCount, D3DXMATRIX worldMatrix, D3DXMATRIX viewMatrix, 
					D3DXMATRIX projectionMatrix, ID3D11ShaderResourceView* texture, D3DXVECTOR3 lightDirection, D3DXVECTOR4 diffuseColor);
		void Shutdown();

private:
		bool InitializeShader(ID3D11Device* device, HWND hwnd, WCHAR* vsFilename, WCHAR* psFilename, LPCSTR vsFunction, LPCSTR vsProfileTarget, LPCSTR psFunction, LPCSTR psProfileTarget);
		bool SetShaderParameters(ID3D11DeviceContext* deviceContext, D3DXMATRIX worldMatrix, D3DXMATRIX viewMatrix, 
								 D3DXMATRIX projectionMatrix, ID3D11ShaderResourceView* texture, D3DXVECTOR3 lightDirection, 
								 D3DXVECTOR4 diffuseColor);
		void ShutdownShader();
		void OutputShaderErrorMessage(ID3D10Blob* errorMessage, HWND hwnd, WCHAR* shaderFilename);
		void RenderShader(ID3D11DeviceContext* deviceContext, int indexCount);

private:

	struct DefaultMatrixBufferType
	{
		D3DXMATRIX world;
		D3DXMATRIX view;
		D3DXMATRIX projection;
	};

	struct LightBufferType
	{
		D3DXVECTOR4 diffuseColor;
		D3DXVECTOR3 lightDirection;
		float padding; 
	};

	ID3D11VertexShader* m_vertexShader;
	ID3D11PixelShader* m_pixelShader;
	ID3D11InputLayout* m_layout; //geometry primitive input data
	ID3D11Buffer* m_matrixBuffer; //WVP matrix data
	ID3D11Buffer* m_lightBuffer; //light direction data
	ID3D11SamplerState* m_sampleState; //texture data
};

#endif