////////////////////////////////////
// Camera class implementation.
// Version: 090113_01
////////////////////////////////////

#include "camera.h"

//Debug only
#include <iostream>
using namespace std;

Camera::Camera()
{
	m_vEyePt	= D3DXVECTOR3(0.0f,0.0f,-50.0f);
	m_vLookAtPt = D3DXVECTOR3(0.0f,0.0f,0.0f);
	m_vUp		= D3DXVECTOR3(0.0f,1.0f,0.0f);

	m_speed			= 0.05f;
	m_turnSpeed		= 1.5f;
	m_rollSpeed		= 0.005f;
	m_pitchSpeed	= 0.5f;
}

Camera::~Camera()
{

}

void Camera::MoveForward()
{
	D3DXVec3Normalize(&vDirection,&(m_vLookAtPt - m_vEyePt));
	m_vEyePt += vDirection * m_speed;
	m_vLookAtPt += vDirection * m_speed;
}

void Camera::MoveBack()
{
	D3DXVec3Normalize(&vDirection,&(m_vLookAtPt - m_vEyePt));
	m_vEyePt -= vDirection * m_speed;
	m_vLookAtPt -= vDirection * m_speed;
}

void Camera::StrafeLeft()
{
	D3DXVec3Cross(&vDirection,&vDirection, &m_vUp); 
	D3DXVec3Normalize(&vDirection, &vDirection);

	m_vEyePt += vDirection * m_turnSpeed;
    m_vLookAtPt += vDirection * m_turnSpeed;
}

void Camera::StrafeRight()
{
	D3DXVec3Cross(&vDirection,&vDirection, &m_vUp); 
	D3DXVec3Normalize(&vDirection, &vDirection);

	m_vEyePt -= vDirection * m_turnSpeed;
    m_vLookAtPt -= vDirection * m_turnSpeed;
}

void Camera::Rotate(float xAxis, float yAxis)
{
	D3DXVECTOR3 vRotAxis;
	D3DXMATRIX matRotAxis,matRotY;

	D3DXVec3Normalize(&vDirection, &(m_vLookAtPt - m_vEyePt)); 

	D3DXVec3Cross(&vRotAxis,&vDirection,&m_vUp); 
	D3DXVec3Normalize(&vRotAxis,&vRotAxis);

	D3DXMatrixRotationAxis(&matRotAxis, &vRotAxis,  yAxis / -360);
	D3DXMatrixRotationY(&matRotY, xAxis / -360);

	D3DXVec3TransformCoord(&vDirection,&vDirection,&(matRotAxis * matRotY));
	D3DXVec3TransformCoord(&m_vUp,&m_vUp,&(matRotAxis * matRotY));
	m_vLookAtPt = vDirection + m_vEyePt;
}

void Camera::Update()
{
	D3DXMatrixLookAtLH(&m_viewMatrix,&m_vEyePt,&m_vLookAtPt,&m_vUp);
}


////////////////////////
//Flightsim specific
////////////////////////

void Camera::RotateRoll(float xAxis, float yAxis)
{
	D3DXVECTOR3 vRotAxis;
	D3DXMATRIX matRotAxis,matRotY;

	D3DXVec3Normalize(&vDirection, &(m_vLookAtPt - m_vEyePt)); 

	D3DXVec3Cross(&vRotAxis,&vDirection,&m_vUp); 
	D3DXVec3Normalize(&vRotAxis,&vRotAxis);

	D3DXMatrixRotationAxis(&matRotAxis, &vRotAxis,  yAxis / -360);
	D3DXMatrixRotationZ(&matRotY, xAxis / -360);

	D3DXVec3TransformCoord(&vDirection,&vDirection,&(matRotAxis * matRotY));
	D3DXVec3TransformCoord(&m_vUp,&m_vUp,&(matRotAxis * matRotY));
	m_vLookAtPt = vDirection + m_vEyePt;
}

void Camera::RollLeft()
{
	float roll = 0.0f;
	D3DXMATRIX rotationMatrix;

	roll  += m_rollSpeed;

	D3DXMatrixRotationYawPitchRoll(&rotationMatrix, 0, 0, roll);

	D3DXVec3TransformCoord(&m_vLookAtPt, &m_vLookAtPt, &rotationMatrix);
	D3DXVec3TransformCoord(&m_vUp, &m_vUp, &rotationMatrix);

	m_vLookAtPt = vDirection + m_vEyePt;
}

void Camera::RollRight()
{
	float roll = 0.0f;
	D3DXMATRIX rotationMatrix;

	roll -= m_rollSpeed;

	D3DXMatrixRotationYawPitchRoll(&rotationMatrix, 0, 0, roll);

	D3DXVec3TransformCoord(&m_vLookAtPt, &m_vLookAtPt, &rotationMatrix);
	D3DXVec3TransformCoord(&m_vUp, &m_vUp, &rotationMatrix);

	m_vLookAtPt = vDirection + m_vEyePt;
}

void Camera::YawLeft()
{
	D3DXVECTOR3 vRotAxis;
	D3DXMATRIX matRotAxis,matRotY;

	D3DXVec3Normalize(&vDirection, &(m_vLookAtPt - m_vEyePt)); 

	D3DXVec3Cross(&vRotAxis,&vDirection,&m_vUp); 
	D3DXVec3Normalize(&vRotAxis,&vRotAxis);

	D3DXMatrixRotationAxis(&matRotAxis, &vRotAxis, 0);
	D3DXMatrixRotationY(&matRotY, m_turnSpeed / -360);

	D3DXVec3TransformCoord(&vDirection,&vDirection,&(matRotY));
	D3DXVec3TransformCoord(&m_vUp,&m_vUp,&(matRotY));

	m_vLookAtPt = vDirection + m_vEyePt;
}

void Camera::YawRight()
{
	D3DXVECTOR3 vRotAxis;
	D3DXMATRIX matRotAxis,matRotY;

	D3DXVec3Normalize(&vDirection, &(m_vLookAtPt - m_vEyePt)); 

	D3DXVec3Cross(&vRotAxis,&vDirection,&m_vUp); 
	D3DXVec3Normalize(&vRotAxis,&vRotAxis);

	D3DXMatrixRotationAxis(&matRotAxis, &vRotAxis, 0);
	D3DXMatrixRotationY(&matRotY, -m_turnSpeed / -360);

	D3DXVec3TransformCoord(&vDirection,&vDirection,&(matRotY));
	D3DXVec3TransformCoord(&m_vUp,&m_vUp,&(matRotY));

	m_vLookAtPt = vDirection + m_vEyePt;
}

//Never implemented this because I decided to use a quaternion based camera instead.
void Camera::PitchUp()
{
	
}

void Camera::PitchDown()
{

}

