////////////////////////////////////////////////////////////////////////////////////////////
// Euler angle matrix rotation based camera class.
// This class controls the projection of the scene from a first person point of view.
// It also allows the viewer to move and rotate around the scene. 
//
// Version: 090113_01
////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _CAMERA_H_
#define _CAMERA_H_

#include <d3dx10math.h>

class Camera
{
public:

	Camera();
	~Camera();

	void Update();
	void MoveForward();
	void MoveBack();
	void StrafeLeft();
	void StrafeRight();
	void YawLeft();
	void YawRight();
	void RollLeft();
	void RollRight();
	void PitchUp();
	void PitchDown();
	void Rotate(float xAxis, float yAxis);
	void RotateRoll(float xAxis, float yAxis);

	inline void GetViewMatrix(D3DXMATRIX& viewMatrix) {viewMatrix = m_viewMatrix;}

	inline void SetPosition(float x, float y, float z) 
	{
		m_vEyePt.x = x;
		m_vEyePt.y = y;
		m_vEyePt.z = z;
	}

	////////////////////////////////////////
	//Flight mode prototype functions
	////////////////////////////////////////
	inline void AccelerateForward(float delta)
	{
		D3DXVec3Normalize(&vDirection,&(m_vLookAtPt - m_vEyePt));
		m_vEyePt += vDirection * m_speed;
		m_vLookAtPt += vDirection * (m_speed * delta);
	}

public:

	float m_speed;
	float m_turnSpeed;
	float m_rollSpeed;
	float m_pitchSpeed;

private:

	D3DXVECTOR3  m_vEyePt, m_vLookAtPt, m_vUp;
	D3DXVECTOR3	 vDirection;
	D3DXMATRIX	 m_viewMatrix;

};

#endif