//////////////////////////////////////////////////////////////////////////////////////////////////
// Cube model scene component class.
// This class is used to test basic rendering of a textured and animated primitive 3D object.
// (In this case a single directional light lit textured cube spinning around it's y-axis). 
//
// Version: 201212_01
//////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _CUBUE_H_
#define _CUBE_H_

#include <d3d11.h>
#include <d3dx10math.h>
#include <d3dx11tex.h>
#include <fstream>

using namespace std;

class CubeModel
{
public:
		CubeModel();
		~CubeModel();

		bool Init(ID3D11Device* device, char* modelFilename, WCHAR* textureFilename);
		void Render(ID3D11DeviceContext* deviceContext);
		void Shutdown();

		inline int GetIndexCount() {return m_indexCount;}
		inline ID3D11ShaderResourceView* GetTexture() {return m_texture;}

private:
		bool InitializeBuffers(ID3D11Device* device);
		void ShutdownBuffers();
		void RenderBuffers(ID3D11DeviceContext* deviceContext);

		bool LoadTexture(ID3D11Device* device, WCHAR* filename);
		void ReleaseTexture();

		bool LoadModel(char* filename);
		void ReleaseModel();

private:

	struct VertexType
	{
		D3DXVECTOR3 position;
	    D3DXVECTOR2 texture;
		D3DXVECTOR3 normal;
	};

	struct ModelType
	{
		float x, y, z;
		float tu, tv;
		float nx, ny, nz;
	};

	ID3D11Buffer *m_vertexBuffer, *m_indexBuffer;
	ID3D11ShaderResourceView* m_texture;
	int m_vertexCount, m_indexCount;
	ModelType* m_model;
};

#endif