///////////////////////////////////////////////////////////////////
// DirectX 11 - Direct3D device rendering context class.
// Thsi class handles the creation, updating and cleanup of the
// D3D device context within the framework.
// 
// Version: 110113_01
///////////////////////////////////////////////////////////////////

#ifndef _D3D_SYSTEM_H_
#define _D3D_SYSTEM_H_

//DirectX 11 system includes and libs linking
#pragma comment(lib, "dxgi.lib")
#pragma comment(lib, "d3d11.lib")
#pragma comment(lib, "d3dx11.lib")
#pragma comment(lib, "d3dx10.lib")

#include <dxgi.h>
#include <d3dcommon.h>
#include <d3d11.h>
#include <d3dx10math.h>

class D3DSystem
{
public:

	D3DSystem();
	~D3DSystem();

	bool Init(int screenWidth, int screenHeight, bool vsync, HWND hwnd, bool fullscreen, float screenDepth, float screenNear);
	void BeginScene(float red, float green, float blue, float alpha);
	void EndScene();
	void ShutDown();

	ID3D11Device* GetDevice();
	ID3D11DeviceContext* GetDeviceContext();

	void GetProjectionMatrix(D3DXMATRIX&);
	void GetWorldMatrix(D3DXMATRIX&);
	void GetOrthoMatrix(D3DXMATRIX&);

	void GetVideoCardInfo(char* cardName, int& memory);

	inline void DisableDepthBuffer()
	{
		m_deviceContext->OMSetDepthStencilState(m_disabledDepthStencilState, 1);
	}

	inline void EnableDepthBuffer()
	{
		m_deviceContext->OMSetDepthStencilState(m_depthStencilState, 1);
	}

private:
	bool m_vsync_enabled;
	int m_videoCardMemory;
	char m_videoCardDescription[128];

public:
	IDXGISwapChain* m_swapChain;
	ID3D11Device* m_device;
	ID3D11DeviceContext* m_deviceContext;
	ID3D11RenderTargetView* m_renderTargetView;
	ID3D11Texture2D* m_depthStencilBuffer;
	ID3D11DepthStencilState* m_depthStencilState;
	ID3D11DepthStencilState* m_disabledDepthStencilState;
	ID3D11DepthStencilView* m_depthStencilView;
	ID3D11RasterizerState* m_rasterState;
	D3D11_TEXTURE2D_DESC depthBufferDesc;
	D3DXMATRIX m_projectionMatrix;
	D3DXMATRIX m_worldMatrix;
	D3DXMATRIX m_orthoMatrix;
};


#endif