/////////////////////////////////////////////
// Graphics system class implementation.
// Version: 150113_01
////////////////////////////////////////////

#include "graphicsSystem.h"
#include <math.h>

//randomizer
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

//debug
#include <iostream>
#include <string>
#include <conio.h>
using namespace std;

GraphicsSystem::GraphicsSystem()
{
	m_D3DSystem = NULL;
	m_quatCam = NULL;
	m_OrangeBeacon = NULL;
	m_ColorShader = NULL;
	m_Input = NULL;
	m_Grid = NULL;
	m_SystemTimer = NULL;
	m_FontRenderer = NULL;
	m_GameLogo = NULL;

	for(int i =0; i < 512; ++i)
	{
		m_Clouds[i] = NULL;
	}

	b_quitApp = false;

	//Initialize all of the game variables
	bLandingGear = true;
	bFlaps = false;
	bInitialTakeOff = true;
	bOffCourse = false;

	//set the initial enum states
	mGameState = INTRO;
	planeState = TAXI;

	srand(time(NULL));

	//Set the cloud positions
	for(int j = 0; j < 512; ++j)
	{
		cloudPositions[j].x = rand() % 6400 + 1;
		cloudPositions[j].y = rand() % 8 + 512;
		cloudPositions[j].z = rand() % 6400 + 1;
		cloudScale[j] =  rand() % 5 + 1;
	}
}

GraphicsSystem::~GraphicsSystem()
{

}

bool GraphicsSystem::Init(HINSTANCE hInstance, int screenWidth, int screenHeight, HWND hwnd)
{
	//Create and initialize the D3D rendering context object here
	bool result;

	m_D3DSystem = new D3DSystem;

	if(!m_D3DSystem)
	{
		return false;
	}

	// Initialize the Direct3D object.
	result = m_D3DSystem->Init(screenWidth, screenHeight, VSYNC_ENABLED, hwnd, FULL_SCREEN, SCREEN_DEPTH, SCREEN_NEAR);

	if(!result)
	{
		MessageBox(hwnd, L"Could not initialize Direct3D.", L"Error", MB_OK);
		return false;
	}

	//Init the system timer
	m_SystemTimer = new Timer();
	if(!m_SystemTimer)
	{
		return false;
	}

	//Create & init the quaternion camera object
	m_quatCam = new QuaternionCamera();
	m_quatCam->InitCamera();

	//Init the input
	m_Input = new Input();
	m_Input->Init(hInstance, hwnd, screenWidth, screenHeight);
	
	// Create the model object.
	m_OrangeBeacon = new Primitive;
	if(!m_OrangeBeacon)
	{
		return false;
	}

	m_GreenBeacon = new Primitive;
	if(!m_GreenBeacon)
	{
		return false;
	}

	m_Grid = new Grid();
	m_Grid->Init(m_D3DSystem->GetDevice());

	//clouds
	D3DXVECTOR4 whiteTriangleColor = D3DXVECTOR4(1.0f, 1.8f, 1.0f, 1.0f);
	for(int i =0; i < 512; ++i)
	{
		m_Clouds[i] = new Primitive;
		m_Clouds[i]->Init(m_D3DSystem->GetDevice(), &whiteTriangleColor);
	}

	// Initialize the beacon models
	D3DXVECTOR4 orangeTriangleColor = D3DXVECTOR4(1.0, 0.4, 0.0, 1.0);

	result = m_OrangeBeacon->Init(m_D3DSystem->GetDevice(), &orangeTriangleColor);

	if(!result)
	{
		MessageBox(hwnd, L"Could not initialize the model object.", L"Error", MB_OK);
		return false;
	}

	D3DXVECTOR4 greenTriangleColor = D3DXVECTOR4(0.0f, 0.8f, 0.0f, 1.0f);

	result = m_GreenBeacon->Init(m_D3DSystem->GetDevice(), &greenTriangleColor);

	if(!result)
	{
		MessageBox(hwnd, L"Could not initialize the model object.", L"Error", MB_OK);
		return false;
	}

	// Create the color shader object.
	m_ColorShader = new Shader;
	if(!m_ColorShader)
	{
		return false;
	}

	// Initialize the color shader object.
	result = m_ColorShader->Initialize(m_D3DSystem->GetDevice(), hwnd);
	if(!result)
	{
		MessageBox(hwnd, L"Could not initialize the color shader object.", L"Error", MB_OK);
		return false;
	}

	//Setup the font renderer
	m_FontRenderer = new FontClass();
	m_FontRenderer->Init(m_D3DSystem->GetDevice());

	m_GameLogo = new Logo();
	m_GameLogo->Init(m_D3DSystem->GetDevice());

	return true;
}

bool GraphicsSystem::ProcessFrame()
{
	bool result;

	if(mGameState == INTRO)
	{
		m_Input->ProcessFrame();

		if(m_Input->m_keyboardState[DIK_SPACE] & 0x80)
		{
			mGameState = RUNNING;
		}

		if(m_Input->m_keyboardState[DIK_ESCAPE] & 0x80)
		{
			b_quitApp = true;
		}

		IntroScreen();
	}

	if(mGameState == GAME_OVER)
	{
		m_Input->ProcessFrame();

		if(m_Input->m_keyboardState[DIK_ESCAPE] & 0x80)
		{
			b_quitApp = true;
		}

		GameOver();
	}


	if(mGameState == RUNNING)
	{
		////////////////////////
		//Update the timer
		////////////////////////
		m_SystemTimer->Update();

		//////////////////////////////
		//Update the flight camera
		//////////////////////////////
		m_quatCam->UpdateCamera();

		///////////////////////////////////////
		//Update the flight simulation here
		///////////////////////////////////////
		m_quatCam->StepSimulation(m_SystemTimer->GetTime());

		///////////////////////////////////////////////////////////////////
		//Flight mode camera controls (Quaternion angle rotation based).
		///////////////////////////////////////////////////////////////////
		if(planeState != CRASHED)
		{
			if(m_Input->m_keyboardState[DIK_A] & 0x80)
			{
				m_quatCam->YawLeft();
			}
			if(m_Input->m_keyboardState[DIK_D] & 0x80)
			{
				m_quatCam->YawRight();
			}
		}

		if(planeState != TAXI && planeState != CRASHED)
		{
			if(m_Input->m_keyboardState[DIK_LEFTARROW] & 0x80)
			{
				m_quatCam->RollLeft();
			}
			if(m_Input->m_keyboardState[DIK_RIGHTARROW] & 0x80)
			{
				m_quatCam->RollRight();
			}
		}

		//Pitch up/down
		if(planeState != CRASHED)
		{
			if(m_quatCam->m_speed > 0.045f)
			{
				if(planeState == IN_FLIGHT)
				{
					if(m_Input->m_keyboardState[DIK_UPARROW] & 0x80)
					{
						m_quatCam->PitchUp();
					}
				}

				if(m_Input->m_keyboardState[DIK_DOWNARROW] & 0x80)
				{
					m_quatCam->PitchDown();
				}
			}
		}

		//////////////////
		// Crash landing
		//////////////////
		if((m_quatCam->m_speed > 0.15f && m_quatCam->m_vEyePt.y < 0.06f) && !bInitialTakeOff)
		{
			m_quatCam->m_vEyePt.y = 0.04f;
			planeState = CRASHED;
		}

		//if roll is too high, altitude is ground level and !bInitialTakeOff
		if((m_quatCam->roll < -0.25f || m_quatCam->roll > 0.25f) &&  m_quatCam->m_vEyePt.y < 0.06f && !bInitialTakeOff)
		{
			planeState = CRASHED;
		}

		//if pitch is too high, altitude is ground level and !bInitialTakeOff
		if((m_quatCam->pitch > 0.3f) &&  m_quatCam->m_vEyePt.y < 0.06f && !bInitialTakeOff)
		{
			planeState = CRASHED;
		}
		

		if(planeState == TAXI && !bLandingGear)
		{
			planeState = CRASHED;
		}

		//////////////////////////////
		//Update the plane states
		//////////////////////////////
		if(m_quatCam->m_vEyePt.y > 0.06f)
		{
			planeState = IN_FLIGHT;
		}
		else if(m_quatCam->m_speed < 0.01f && m_quatCam->m_vEyePt.y < 0.06f && (planeState != CRASHED) && (planeState != STALL))
		{
			planeState = TAXI;
		}

		if(planeState == IN_FLIGHT || m_quatCam->m_vEyePt.y > 0.05f)
		{
			bInitialTakeOff = false;
		}

		//Stall
		if((m_quatCam->m_speed < 0.1f && m_quatCam->pitch < -0.785f) && planeState != TAXI)
		{
			planeState = STALL;
		}

		if(m_quatCam->m_vEyePt.y > 500.0f)
		{
			planeState = STALL;
		}

		if(planeState == STALL)
		{
			m_quatCam->Stall(m_SystemTimer->GetTime());
		}

		if(planeState == STALL && m_quatCam->m_vEyePt.y < 0.06f)
		{
			planeState = CRASHED;
		}

		//Off course
		if(m_quatCam->m_vEyePt.x < 0.0f || m_quatCam->m_vEyePt.x > 6400.0f)
		{
			bOffCourse = true;
		}
		else if(m_quatCam->m_vEyePt.z < -15.0f || m_quatCam->m_vEyePt.z > 6400.0f)
		{
			bOffCourse = true;
		}
		else
		{
			bOffCourse = false;
		}

		//Landed succesfully (ideal landing conditions)
		if(m_quatCam->m_vEyePt.x > 6080.0f && m_quatCam->m_vEyePt.x < 6400.0f)
		{
			if(m_quatCam->m_vEyePt.z > 6080.0f && m_quatCam->m_vEyePt.z < 6400.0f)
			{
				if(m_quatCam->m_speed < 0.075f && (m_quatCam->roll > -0.15f && m_quatCam->roll < 0.15f))
				{
					if(m_quatCam->pitch < 0.3f && m_quatCam->pitch > -0.3f)
					{
						if(bLandingGear)
						{
							planeState = LANDED;
						}
					}
				}
			}
		}

		//Non ideal landing conditions, will cause crash
		if(planeState == LANDED)
		{
			if((m_quatCam->roll < -0.25f || m_quatCam->roll > 0.25f) || !bLandingGear || (m_quatCam->pitch > 0.3f || m_quatCam->pitch < -0.3f))
			{
				planeState = CRASHED;
			}
		}

		//Increase/decrease thrust
		if(m_Input->m_keyboardState[DIK_Q] & 0x80)
		{
			m_quatCam->IncreaseThrust();
		}
		if(m_Input->m_keyboardState[DIK_E] & 0x80)
		{
			m_quatCam->DecreaseThrust();
		}

		if(bFlaps)
		{
			m_quatCam->Flaps(m_SystemTimer->GetTime());
		}

		///////////////////////////
		//Get aux keyboard input
		///////////////////////////
		if(!m_Input->m_keyboardState[DIK_ESCAPE] & 0x80)
		{
			return false;
		}

		m_Input->ProcessFrame();

		if(m_Input->m_keyboardState[DIK_ESCAPE] & 0x80)
		{
			b_quitApp = true;
		}

		/////////////////////////
		//Flaps & gear control
		/////////////////////////
		if(m_Input->m_keyboardState[DIK_F] & 0x80)
		{
			Sleep(100);
			bFlaps = !bFlaps;
		}
		if(m_Input->m_keyboardState[DIK_G] & 0x80)
		{
			Sleep(100);
			bLandingGear = !bLandingGear;
		}

		/////////////////////
		//Render the scene
		/////////////////////
		result = Render();

		if(!result)
		{
			return false;
		}

		////////////////////////////////////////////////////////////////////////////
		// Console window output for speed, altitude, yaw, pitch and roll values
		////////////////////////////////////////////////////////////////////////////
		cout << string( 100, '\n' ); //very very hacky, but its a really quick way to clear the console screen (much quicker then using system()).
		cout << "Speed: "		<< m_quatCam->m_speed * 1000.0f << "km/h" << endl;
		cout << "Altitude: "	<< m_quatCam->m_vEyePt.y << "m" <<endl;
		cout << "Yaw: "			<< m_quatCam->yaw * (57.29577951) << " degrees"  << endl;
		cout << "Pitch: "		<< m_quatCam->pitch * (57.29577951) << " degrees"  << endl;
		cout << "Roll: "		<< m_quatCam->roll * (57.29577951) << " degrees"  << endl;
		cout << "X-coordinate position: "	<< m_quatCam->m_vEyePt.x << endl;
		cout << "Y-coordinate position: "	<< m_quatCam->m_vEyePt.z << endl;
	}

	return true;
}

void GraphicsSystem::ShutDown()
{
	if(m_D3DSystem)
	{
		m_D3DSystem->ShutDown();
		delete m_D3DSystem;
		m_D3DSystem = 0;
	}

	if(m_ColorShader)
	{
		m_ColorShader->Shutdown();
		delete m_ColorShader;
		m_ColorShader = 0;
	}

	if(m_OrangeBeacon)
	{
		m_OrangeBeacon->Shutdown();
		delete m_OrangeBeacon;
	}

	if(m_GreenBeacon)
	{
		m_GreenBeacon->Shutdown();
		delete m_GreenBeacon;
	}

	delete m_quatCam;

	m_Grid->Shutdown();

	m_Input->Shutdown();

	m_FontRenderer->Release();

	m_GameLogo->Release();

	delete m_SystemTimer;
}

void GraphicsSystem::IntroScreen()
{
	m_D3DSystem->DisableDepthBuffer();

	m_D3DSystem->BeginScene(0.2f, 0.4f, 1.0f, 1.0f);

	m_GameLogo->Render(m_D3DSystem->GetDeviceContext(), m_D3DSystem->m_swapChain, m_D3DSystem->m_renderTargetView);

	m_FontRenderer->Render(m_D3DSystem->GetDeviceContext(), m_D3DSystem->m_swapChain, m_D3DSystem->m_renderTargetView,  "PRESS SPACE TO START", -0.4f, 0.9f);

	m_D3DSystem->EndScene();

	m_D3DSystem->EnableDepthBuffer();
}

void GraphicsSystem::GameOver()
{
	m_D3DSystem->DisableDepthBuffer();

	m_D3DSystem->BeginScene(0.2f, 0.4f, 1.0f, 1.0f);

	m_FontRenderer->Render(m_D3DSystem->GetDeviceContext(), m_D3DSystem->m_swapChain, m_D3DSystem->m_renderTargetView,  "GAME OVER", -0.15f, 0.0f);

	m_D3DSystem->EndScene();

	m_D3DSystem->EnableDepthBuffer();
}

void GraphicsSystem::RenderGUI()
{
	m_D3DSystem->DisableDepthBuffer();

	if(planeState == TAXI)
	{
		m_FontRenderer->Render(m_D3DSystem->GetDeviceContext(), m_D3DSystem->m_swapChain, m_D3DSystem->m_renderTargetView,  "TAXI MODE", -0.2f, 0.8f);
	}
	else
	{
		m_FontRenderer->Render(m_D3DSystem->GetDeviceContext(), m_D3DSystem->m_swapChain, m_D3DSystem->m_renderTargetView,  "FLIGHT MODE", -0.2f, 0.8f);
	}

	if(planeState == CRASHED)
	{
		m_FontRenderer->Render(m_D3DSystem->GetDeviceContext(), m_D3DSystem->m_swapChain, m_D3DSystem->m_renderTargetView,  "PLANE CRASH", -0.2f, 0.8f);

			//Game Over
			Sleep(2000);

			mGameState = GAME_OVER;
	}

	if(bOffCourse)
	{
		m_FontRenderer->Render(m_D3DSystem->GetDeviceContext(), m_D3DSystem->m_swapChain, m_D3DSystem->m_renderTargetView,  "OFF-COURSE ", -0.2f, 0.8f);
	}

	if(planeState == LANDED)
	{
		m_FontRenderer->Render(m_D3DSystem->GetDeviceContext(), m_D3DSystem->m_swapChain, m_D3DSystem->m_renderTargetView,  "LANDED", -0.2f, 0.8f);

		//Game Over
		Sleep(3000);

		mGameState = GAME_OVER;
	}

	if(planeState == STALL)
	{
		m_FontRenderer->Render(m_D3DSystem->GetDeviceContext(), m_D3DSystem->m_swapChain, m_D3DSystem->m_renderTargetView,  "STALL WARNING", -0.2f, 0.8f);
	}

	if(bFlaps)
	{
		m_FontRenderer->Render(m_D3DSystem->GetDeviceContext(), m_D3DSystem->m_swapChain, m_D3DSystem->m_renderTargetView,  "FLAPS", -0.8f, -0.6f);
	}
	if(bLandingGear)
	{
		m_FontRenderer->Render(m_D3DSystem->GetDeviceContext(), m_D3DSystem->m_swapChain, m_D3DSystem->m_renderTargetView,  "GEAR", 0.6f, -0.6f);
	}

	m_D3DSystem->EnableDepthBuffer();
}

bool GraphicsSystem::Render()
{
	D3DXMATRIX viewMatrix, projectionMatrix, worldMatrix, tansMatrix, scaleMatrix;

	// Clear the buffers to begin the scene.
	m_D3DSystem->BeginScene(0.2f, 0.4f, 1.0f, 1.0f);

	m_quatCam->GetViewMatrix(viewMatrix);
	m_D3DSystem->GetWorldMatrix(worldMatrix);
	m_D3DSystem->GetProjectionMatrix(projectionMatrix);

	//Draw the orange beacon
	D3DXMatrixTranslation(&worldMatrix, 160, 1, 160); 
	m_OrangeBeacon->Render(m_D3DSystem->GetDeviceContext());
	m_ColorShader->Render(m_D3DSystem->GetDeviceContext(), m_OrangeBeacon->GetIndexCount(), worldMatrix, viewMatrix, projectionMatrix);
	D3DXMatrixIdentity(&worldMatrix);

	//Draw the green beacon
	D3DXMatrixTranslation(&worldMatrix, 6240, 1, 6240); 
	m_GreenBeacon->Render(m_D3DSystem->GetDeviceContext());
	m_ColorShader->Render(m_D3DSystem->GetDeviceContext(), m_GreenBeacon->GetIndexCount(), worldMatrix, viewMatrix, projectionMatrix);
	D3DXMatrixIdentity(&worldMatrix);

	//Draw the grid area
	D3DXMatrixScaling(&scaleMatrix, 10.0f, 1.0f, 10.0f);
	D3DXMatrixMultiply(&worldMatrix, &worldMatrix, &scaleMatrix); 
	m_Grid->Render(m_D3DSystem->GetDeviceContext());
	m_ColorShader->Render(m_D3DSystem->GetDeviceContext(), m_Grid->GetIndexCount(), worldMatrix, viewMatrix, projectionMatrix);
	D3DXMatrixIdentity(&worldMatrix);

	//Draw the clouds (using a billboarded projection matrix)
	for(int i =0; i < 512; ++i)
	{
		float angle = atan2(cloudPositions[i].x - m_quatCam->m_vEyePt.x, cloudPositions[i].z - m_quatCam->m_vEyePt.z) * (180.0 / D3DX_PI);
		float rotation = (float)angle * 0.0174532925f;	

		D3DXMatrixRotationY(&worldMatrix, rotation);
		D3DXMatrixScaling(&scaleMatrix, cloudScale[i], cloudScale[i], cloudScale[i]);
		D3DXMatrixMultiply(&worldMatrix, &worldMatrix, &scaleMatrix); 
		D3DXMatrixTranslation(&tansMatrix, cloudPositions[i].x, cloudPositions[i].y, cloudPositions[i].z);
		D3DXMatrixMultiply(&worldMatrix, &worldMatrix, &tansMatrix); 

		m_Clouds[i]->Render(m_D3DSystem->GetDeviceContext());
		m_ColorShader->Render(m_D3DSystem->GetDeviceContext(), m_Clouds[i]->GetIndexCount(), worldMatrix, viewMatrix, projectionMatrix);

		D3DXMatrixIdentity(&worldMatrix);
	}

	//Render the GUI
	RenderGUI();
	
	m_D3DSystem->EndScene();

	return true;
}

