//////////////////////////////////////////////////////////////////////
// Graphics system class.
// Handles the Direct 3D specific device context rendering
// within the application window, either windowed or fullscreen.
//
// Version: 150113_01
//////////////////////////////////////////////////////////////////////

#ifndef _GRAPHICS_SYSTEM_H_
#define _GRAPHICS_SYSTEM_H_

//include d3d class
#include "d3dSystem.h"
#include "primitive.h"
#include "shader.h"
#include "input.h"
#include "grid.h"
#include "timer.h"
#include "quatCamera.h"
#include "font.h"
#include "logo.h"

//Global constant defines
const bool FULL_SCREEN = false;
const bool VSYNC_ENABLED = true;
const float SCREEN_DEPTH = 10000.0f;
const float SCREEN_NEAR = 0.1f;

class GraphicsSystem
{
public:

	GraphicsSystem();
	~GraphicsSystem();

	bool Init(HINSTANCE hInstance, int screenWidth, int screenHeight, HWND hwnd);
	bool ProcessFrame();
	void RenderGUI();
	void IntroScreen();
	void GameOver();
	void ShutDown();

private:

	bool Render();

public:

	bool b_quitApp;

private:

	//D3D class instance
	D3DSystem*	m_D3DSystem;
	QuaternionCamera* m_quatCam;
	Primitive*  m_OrangeBeacon;
	Primitive*	m_GreenBeacon;
	Primitive*	m_Clouds[512];
	Shader*		m_ColorShader;
	Input*		m_Input;
	Grid*		m_Grid;
	Timer*		m_SystemTimer;
	FontClass*  m_FontRenderer;
	Logo*		m_GameLogo;

private:

	//Game control variables
	bool	bLandingGear;
	bool	bFlaps;
	bool	bInitialTakeOff;
	bool	bOffCourse;
	float	distanceFromTarget;

	//Game states
	enum gameState{INTRO, RUNNING, GAME_OVER};
	gameState mGameState;
	
	//Flight state enums
	enum flightState{TAXI, IN_FLIGHT, STALL, LANDED, CRASHED};
	flightState planeState;

	//random cloud positions and scale
	D3DXVECTOR3 cloudPositions[512];
	float		cloudScale[512];
};

#endif