/////////////////////////////////////
// Game logo class implementation.
// Version: 150113_01
/////////////////////////////////////

#include "logo.h"

struct VertexPos
{
    XMFLOAT3 pos;
    XMFLOAT2 tex0;
};


void Logo::Init(ID3D11Device* d3dDevice)
{
	ID3DBlob* vsBuffer = 0;

    CompileD3DShader( L"Shaders/font.fx", "VS_Main", "vs_4_0", &vsBuffer );

    HRESULT d3dResult;

    d3dResult = d3dDevice->CreateVertexShader( vsBuffer->GetBufferPointer( ), vsBuffer->GetBufferSize( ), 0, &solidColorVS_ );


    D3D11_INPUT_ELEMENT_DESC solidColorLayout[] =
    {
        { "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D11_INPUT_PER_VERTEX_DATA, 0 },
        { "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 12, D3D11_INPUT_PER_VERTEX_DATA, 0 }
    };

    unsigned int totalLayoutElements = ARRAYSIZE( solidColorLayout );

    d3dDevice->CreateInputLayout( solidColorLayout, totalLayoutElements, vsBuffer->GetBufferPointer( ), vsBuffer->GetBufferSize( ), &inputLayout_ );

    vsBuffer->Release( );


    ID3DBlob* psBuffer = 0;

    CompileD3DShader( L"Shaders/font.fx", "PS_Main", "ps_4_0", &psBuffer );


    d3dResult = d3dDevice->CreatePixelShader( psBuffer->GetBufferPointer( ), psBuffer->GetBufferSize( ), 0, &solidColorPS_ );

    psBuffer->Release( );


	D3DX11CreateShaderResourceViewFromFile( d3dDevice, L"Data/logo.bmp", 0, 0, &colorMap_, 0 );


    D3D11_SAMPLER_DESC colorMapDesc;
    ZeroMemory( &colorMapDesc, sizeof( colorMapDesc ) );
    colorMapDesc.AddressU = D3D11_TEXTURE_ADDRESS_WRAP;
    colorMapDesc.AddressV = D3D11_TEXTURE_ADDRESS_WRAP;
    colorMapDesc.AddressW = D3D11_TEXTURE_ADDRESS_WRAP;
    colorMapDesc.ComparisonFunc = D3D11_COMPARISON_NEVER;
    colorMapDesc.Filter = D3D11_FILTER_MIN_MAG_MIP_LINEAR;
    colorMapDesc.MaxLOD = D3D11_FLOAT32_MAX;

    d3dDevice->CreateSamplerState( &colorMapDesc, &colorMapSampler_ );

  VertexPos vertices[] =
    {
        { XMFLOAT3(  1.0f,  1.0f, 1.0f ), XMFLOAT2( 1.0f, 1.0f ) },
        { XMFLOAT3(  1.0f, -1.0f, 1.0f ), XMFLOAT2( 1.0f, 0.0f ) },
        { XMFLOAT3( -1.0f, -1.0f, 1.0f ), XMFLOAT2( 0.0f, 0.0f ) },

        { XMFLOAT3( -1.0f, -1.0f, 1.0f ), XMFLOAT2( 0.0f, 0.0f ) },
        { XMFLOAT3( -1.0f,  1.0f, 1.0f ), XMFLOAT2( 0.0f, 1.0f ) },
        { XMFLOAT3(  1.0f,  1.0f, 1.0f ), XMFLOAT2( 1.0f, 1.0f ) },
    };

    D3D11_BUFFER_DESC vertexDesc;
    ZeroMemory( &vertexDesc, sizeof( vertexDesc ) );
    vertexDesc.Usage = D3D11_USAGE_DEFAULT;
    vertexDesc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
    vertexDesc.ByteWidth = sizeof( VertexPos ) * 6;

    D3D11_SUBRESOURCE_DATA resourceData;
    ZeroMemory( &resourceData, sizeof( resourceData ) );
    resourceData.pSysMem = vertices;

    d3dResult = d3dDevice->CreateBuffer( &vertexDesc, &resourceData, &vertexBuffer_ );
}

void Logo::Render(ID3D11DeviceContext* d3dContext, IDXGISwapChain* swapChain, ID3D11RenderTargetView* backBufferTarget)
{
	
    unsigned int stride = sizeof( VertexPos );
    unsigned int offset = 0;

    d3dContext->IASetInputLayout( inputLayout_ );
    d3dContext->IASetVertexBuffers( 0, 1, &vertexBuffer_, &stride, &offset );
    d3dContext->IASetPrimitiveTopology( D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST );

    d3dContext->VSSetShader( solidColorVS_, 0, 0 );
    d3dContext->PSSetShader( solidColorPS_, 0, 0 );
    d3dContext->PSSetShaderResources( 0, 1, &colorMap_ );
    d3dContext->PSSetSamplers( 0, 1, &colorMapSampler_ );
	d3dContext->Draw( 6, 0 );

    swapChain->Present( 0, 0 );
}

void Logo::Release()
{
	if( colorMapSampler_ ) colorMapSampler_->Release( );
    if( colorMap_ ) colorMap_->Release( );
    if( solidColorVS_ ) solidColorVS_->Release( );
    if( solidColorPS_ ) solidColorPS_->Release( );
    if( inputLayout_ ) inputLayout_->Release( );
    if( vertexBuffer_ ) vertexBuffer_->Release( );

    colorMapSampler_ = 0;
    colorMap_ = 0;
    solidColorVS_ = 0;
    solidColorPS_ = 0;
    inputLayout_ = 0;
    vertexBuffer_ = 0;
}
