///////////////////////////////////////////////////////////////////////////////////////////////////////
// DirectX 11 Minimal Flightsim Game Prototype.
// Written by Vladeta Stojanovic.
// 
// This application serves as a basic test application for real-time flight sim experimentation.
// All of the rendering is controlled via the programmable pipeline.
//
// Portions of code based on the DirectX 11 tutorials from :
//	- http://www.rastertek.com
// and the books "Beginning DirectX 11 Game Programming" [Sherrod, Jones 2012] and
// "Introduction to 3D Game Programming with DirectX11" [Luna 2012].
//
// Version: 150113_01
///////////////////////////////////////////////////////////////////////////////////////////////////////

#include "system.h"
#include "console.h"

using namespace std;

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR pScmdline, int iCmdshow)
{
	System* gameSystem;
	ConsoleOutput* debugConsole;

	bool result;

	gameSystem = new System();
	debugConsole = new ConsoleOutput();

	debugConsole->ShowConsoleOutput();

	cout << "=================================================" << endl;
	cout << "Minimal Flightsim by Vladeta Stojanovic." << endl;
	cout << "Copyright 2013. All Rights Reserved." << endl;
	cout << "Please see README.TXT for gameplay instructions." << endl;
	cout << "Version 1.0a" << endl;
	cout << "=================================================" << endl;

	if(!gameSystem)
	{
		MessageBox(gameSystem->m_hwnd, L"Unable to create game system instance", L"System Error", NULL);
		return 0;
	}

	result = gameSystem->Init();

	if(result)
	{
		gameSystem->Run();
	}

	gameSystem->ShutDown();
	delete gameSystem;

	return 0;
}