//////////////////////////////////////////////////////////////////////
// Shader class.
// This class is used to handle the compilation and application of 
// HLSL code.
//
// Version: 201212_01
//////////////////////////////////////////////////////////////////////

#ifndef _SHADER_H_
#define _SHADER_H_

#include <d3d11.h>
#include <d3dx10math.h>
#include <d3dx11async.h>
#include <fstream>

using namespace std;

class Shader
{
public:
		Shader();
		~Shader();

		bool Initialize(ID3D11Device* device, HWND hwnd);
		bool SetNewShader(ID3D11Device* device, HWND hwnd, WCHAR* vsFilename, WCHAR* psFilename, LPCSTR vsFunction, LPCSTR vsProfileTarget, LPCSTR psFunction, LPCSTR psProfileTarget);
		void Shutdown();
		bool Render(ID3D11DeviceContext* deviceContext, int indexCount, D3DXMATRIX worldMatrix, D3DXMATRIX viewMatrix, D3DXMATRIX projectionMatrix);

private:
		bool InitializeShader(ID3D11Device* device, HWND hwnd, WCHAR* vsFilename, WCHAR* psFilename, LPCSTR vsFunction, LPCSTR vsProfileTarget, LPCSTR psFunction, LPCSTR psProfileTarget);
		bool SetShaderParameters(ID3D11DeviceContext* deviceContext, D3DXMATRIX worldMatrix, D3DXMATRIX viewMatrix, D3DXMATRIX projectionMatrix);
		void ShutdownShader();
		void OutputShaderErrorMessage(ID3D10Blob* errorMessage, HWND hwnd, WCHAR* shaderFilename);
		void RenderShader(ID3D11DeviceContext* deviceContext, int indexCount);

private:

	struct DefaultMatrixBufferType
	{
		D3DXMATRIX world;
		D3DXMATRIX view;
		D3DXMATRIX projection;
	};

	ID3D11VertexShader* m_vertexShader;
	ID3D11PixelShader* m_pixelShader;
	ID3D11InputLayout* m_layout;
	ID3D11Buffer* m_matrixBuffer;
};

#endif