//////////////////////////////////
// System class implementation.
// Version: 041212_01
//////////////////////////////////

#include "system.h"

System::System()
{
	//Initialize all class instances here
	m_GraphicsSystem = 0;
}

System::~System()
{

}

bool System::Init()
{
	int screenWidth, screenHeight;
	bool result = false;

	screenWidth = 0;
	screenHeight = 0;

	InitializeWindow(screenWidth, screenHeight);

	//Intialize input class

	//Initialize rendering class
	m_GraphicsSystem = new GraphicsSystem;
	if(!m_GraphicsSystem)
	{
		return false;
	}

	m_GraphicsSystem->Init(m_hinstance, screenWidth, screenHeight, m_hwnd);

	return true;
}

void System::Run()
{
	MSG msg;
	bool done, result;

	ZeroMemory(&msg, sizeof(MSG));
	
	done = false;

	while(!done)
	{
		// Handle the windows messages.
		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

		// If windows signals to end the application then exit out.
		if(msg.message == WM_QUIT)
		{
			done = true;
		}
		else
		{
			//Always chech result from last processed frame
			result = ProcessFrame();

			done = m_GraphicsSystem->b_quitApp;
			
		}
	}

	return;
}

void System::ShutDown()
{
	//Cleanup all framework class instances here
	if(m_GraphicsSystem)
	{
		m_GraphicsSystem->ShutDown();
		delete m_GraphicsSystem;
		m_GraphicsSystem = NULL;
	}

	ShutdownWindow();
}

bool System::ProcessFrame()
{
	//Process input
	bool result;

	result = m_GraphicsSystem->ProcessFrame();
	if(!result)
	{
		return false;
	}

	//Process rendering

	return true;
}

LRESULT CALLBACK System::MessageHandler(HWND hwnd, UINT umsg, WPARAM wparam, LPARAM lparam)
{
	//Process any relative events here
	switch(umsg)
	{
		case WM_KEYDOWN:
		{
			return 0;
		}
		case WM_KEYUP:
		{
			return 0;
		}
		default:
		{
			return DefWindowProc(hwnd, umsg, wparam, lparam);
		}
	}
}

void System::InitializeWindow(int& screenWidth, int& screenHeight)
{
	WNDCLASSEX wc;
	DEVMODE dmScreenSettings;
	int posX, posY;

	SystemHandle = this;

	m_hinstance = GetModuleHandle(NULL);

	m_applicationName = L"DirectX11 Framework Test";

	// Setup the windows class with default settings.
	wc.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc   = WndProc;
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;
	wc.hInstance     = m_hinstance;
	wc.hIcon		 = LoadIcon(NULL, IDI_WINLOGO);
	wc.hIconSm       = wc.hIcon;
	wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName  = NULL;
	wc.lpszClassName = m_applicationName;
	wc.cbSize        = sizeof(WNDCLASSEX);
	
	RegisterClassEx(&wc);

	screenWidth  = GetSystemMetrics(SM_CXSCREEN);
	screenHeight = GetSystemMetrics(SM_CYSCREEN);

	if(FULL_SCREEN)
	{
		memset(&dmScreenSettings, 0, sizeof(dmScreenSettings));
		dmScreenSettings.dmSize       = sizeof(dmScreenSettings);
		dmScreenSettings.dmPelsWidth  = (unsigned long)screenWidth;
		dmScreenSettings.dmPelsHeight = (unsigned long)screenHeight;
		dmScreenSettings.dmBitsPerPel = 32;			
		dmScreenSettings.dmFields     = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;

		ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN);

		posX = posY = 0;
	}
	else
	{
		screenWidth  = 1280;
		screenHeight = 768;

		posX = (GetSystemMetrics(SM_CXSCREEN) - screenWidth)  / 2;
		posY = (GetSystemMetrics(SM_CYSCREEN) - screenHeight) / 2;
	}

	m_hwnd = CreateWindowEx(WS_EX_APPWINDOW, m_applicationName, m_applicationName, 
						    WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_POPUP,
						    posX, posY, screenWidth, screenHeight, NULL, NULL, m_hinstance, NULL);

	ShowWindow(m_hwnd, SW_SHOW);
	SetForegroundWindow(m_hwnd);
	SetFocus(m_hwnd);

	//ShowCursor(false);

	return;
}

void System::ShutdownWindow()
{
	// Show the mouse cursor.
	ShowCursor(true);

	if(FULL_SCREEN)
	{
		ChangeDisplaySettings(NULL, 0);
	}

	DestroyWindow(m_hwnd);
	m_hwnd = NULL;

	UnregisterClass(m_applicationName, m_hinstance);

	m_hinstance = NULL;
	SystemHandle = NULL;

	return;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT umessage, WPARAM wparam, LPARAM lparam)
{
	switch(umessage)
	{
		case WM_DESTROY:
		{
			PostQuitMessage(0);
			return 0;
		}
		case WM_CLOSE:
		{
			PostQuitMessage(0);		
			return 0;
		}
		default:
		{
			return SystemHandle->MessageHandler(hwnd, umessage, wparam, lparam);
		}
	}
}


